library("spBayes")

####################################################
## Forest inventory data discussed in Section 4.6
## code for one chain of Model 7 in Section 4.3.
####################################################
## Read in the data points depicted in Figure 5a.
dat <- read.table("Data/modelBABIO.dat", header=TRUE)
coords <- as.matrix(dat[,c("XUTM","YUTM")])

## Set priors, starting, tuning, etc.
K.Psi.prior <- prior(dist="IWISH", df=2, S=matrix(c(100, 0, 0, 2600), 2, 2))
phi.prior <- prior(dist="LOGUNIF", a=0.0015, b=0.03)

K.Psi.starting <- matrix(c(10, 80, 0, 10), 2, 2)

var.update.control <- list("K"=list(sample.order=0, starting=K.Psi.starting,
                             tuning=diag(c(0.15, 1.75, 0.15)), prior=K.Psi.prior),
                           "Psi"=list(sample.order=1, starting=K.Psi.starting,
                             tuning=diag(c(0.15, 1.75, 0.15)), prior=K.Psi.prior),
                           "phi"=list(sample.order=2, starting=0.006,
                             tuning=0.5, prior=list(phi.prior, phi.prior)))

beta.control <- list(update="GIBBS", prior=prior(dist="FLAT"))

## This chain should really be run out for at least 5,000 samples.
run.control <- list("n.samples"=100, "sp.effects"=TRUE, "DIC"=FALSE)

resp.1 <- TOTBA~ELEV+SP_TC1+SP_TC2+SP_TC3
resp.2 <- TOTBIO~ELEV+SP_TC1+SP_TC2+SP_TC3

mod.fit <- ggt.sp(formula=c(resp.1,resp.2), data=dat, run.control=run.control,
              coords=coords, var.update.control=var.update.control,
              beta.update.control=beta.control,
              cov.model="exponential")

plot(mod.fit$p.samples)

##e.g., 1,000 sample burn-in
summary(mcmc(mod.fit$p.samples[1000:nrow(mod.fit$p.samples),]))

## Read in the data points for prediction.
pred.dat <- read.table("Data/holdoutBABIO.dat", header=TRUE)
pred.coords <- pred.dat[,c("XUTM", "YUTM")]

BA.X <- cbind(rep(1, nrow(pred.dat)), pred.dat$ELEV, pred.dat$SP_TC1, pred.dat$SP_TC2, pred.dat$SP_TC3)
BIO.X <- cbind(rep(1, nrow(pred.dat)), pred.dat$ELEV, pred.dat$SP_TC1, pred.dat$SP_TC2, pred.dat$SP_TC3)
 
pred.covars <- mk.mv.X(list(BA.X, BIO.X))

sp.pred <- sp.predict(mod.fit, start=50, thin=5,
                      pred.coords=pred.coords,
                      pred.covars=pred.covars)


## Reproduce Figure 6, again using some surface interpolator (e.g., Akima's interp
## or MBA's mba.surf functions).
library("MBA")
W.1.pred.surf <-
  mba.surf(cbind(pred.coords, rowMeans(sp.pred$pred.sp.effects[seq(1, nrow(sp.pred$pred.sp.effects), 2),])),
	  no.X=200, no.Y=200, extend=FALSE)$xyz.est

W.2.pred.surf <-
  mba.surf(cbind(pred.coords, rowMeans(sp.pred$pred.sp.effects[seq(2, nrow(sp.pred$pred.sp.effects), 2),])),
	   no.X=200, no.Y=200, extend=FALSE)$xyz.est

Y.1.pred.surf <-
  mba.surf(cbind(pred.coords, rowMeans(sp.pred$pred.y[seq(1, nrow(sp.pred$pred.y), 2),])),
	   no.X=200, no.Y=200, extend=FALSE)$xyz.est

Y.2.pred.surf <-
  mba.surf(cbind(pred.coords, rowMeans(sp.pred$pred.y[seq(2, nrow(sp.pred$pred.y), 2),])),
	   no.X=200, no.Y=200, extend=FALSE)$xyz.est

par(mfrow=c(2,2))
image(W.1.pred.surf, main="Figure 6a, predicted random\nspatial effects E[W*|Data]\nfor first response variable.")
contour(W.1.pred.surf, nlevel=5, add=TRUE)
image(W.2.pred.surf, main="Figure 6a, predicted random\nspatial effects E[W*|Data]\nfor second response variable.")
contour(W.2.pred.surf, nlevel=5, add=TRUE)
image(Y.1.pred.surf, main="Figure 6b, first predicted\nresponse E[Y*|Data].")
contour(Y.1.pred.surf, nlevel=5, add=TRUE)
image(Y.2.pred.surf, main="Figure 6b, second predicted\nresponse E[Y*|Data].")
contour(Y.2.pred.surf, nlevel=5, add=TRUE)


